#!/bin/bash

log()
{
	echo "# $* #"
}

exec_cmd()
{
	log "Running [$@]"
	$@
	if [ "$?" -ne 0 ]; then
		log "Failed to run [$@]..."
		if [ -n "${CI}" ]; then
			exit 1
		else
			# Ignore errors for development environment
			log "Ignoring the last error ..."
		fi
	fi
}

generate_compilation_db()
{
	if [ -n "${COMPILATION_FIXUP}" ]; then
		COMPILATION_FIXUP="${COMPILATION_FIXUP/cmake /cmake -DCMAKE_EXPORT_COMPILE_COMMANDS=ON }"
		exec_cmd ${COMPILATION_FIXUP}
	fi

	exec_cmd bear -- make -C ${SOURCE_FOLDER}

	if [ ! -f "compile_commands.json" ]; then
		log "Failed to generate compilation db"
		exit 2
	fi
}

run_cppcheck_validation()
{
	if [ -f "compile_commands.json" ]; then
		exec_cmd cppcheck --error-exitcode=1 --addon=threadsafety --addon=cert -i ./test --inline-suppr ${CPPCHECK_OPTIONS} --project=compile_commands.json
	else
		exec_cmd cppcheck --error-exitcode=1 --addon=threadsafety --addon=cert -i ./test --inline-suppr ${CPPCHECK_OPTIONS} ${SOURCE_FOLDER}
	fi
}

run_cppcheck_clang_validation()
{
	if [ -f "compile_commands.json" ]; then
		exec_cmd cppcheck --error-exitcode=1 --clang -i ./test --inline-suppr ${CPPCHECK_OPTIONS} --project=compile_commands.json
	fi
}

run_infer_analysis()
{
	exec_cmd infer --fail-on-issue --compilation-database compile_commands.json
}

run_flawfinder_checks()
{
    exec_cmd flawfinder --minlevel 4 --error-level=4 ${FLAWFINDER_OPTIONS} ${SOURCE_FOLDER}
}

run_cpd_check()
{
    exec_cmd /usr/local/pmd/bin/run.sh cpd --language c --exclude ./test/ ${CPD_OPTIONS} --files ${SOURCE_FOLDER}
}

main()
{
	log "SOURCE_FOLDER: ${SOURCE_FOLDER}"
	log "COMPILATION_FIXUP: ${COMPILATION_FIXUP}"
	log "CPPCHECK_OPTIONS: ${CPPCHECK_OPTIONS}"
	log "FLAWFINDER_OPTIONS: ${FLAWFINDER_OPTIONS}"
	log "CPD_OPTIONS: ${CPD_OPTIONS:=--minimum-tokens 200}"
	log ""

	if [ -z "${SOURCE_FOLDER}" ]; then
		log "SOURCE_FOLDER not defined"
		if [ -n "${CI}" ]; then
			log "Can't proceed without SOURCE_FOLDER in pipeline"
			exit 5
		else
			log "Assuming current path '.' as SOURCE_FOLDER"
			export SOURCE_FOLDER="."
		fi
	fi

	# Install pre-req if present/defined
	if [ -f "./gitlab-ci/install-dependencies.sh" ]; then
		log "Installing prerequisites for compilation"
		exec_cmd ./gitlab-ci/install-dependencies.sh
	fi

	# Run flawfinder
	run_flawfinder_checks

	# Generate compilation db
	generate_compilation_db

	# TODO: Run gcc fanalyzer analysis 

	# Run cppcheck with compilation-db
	run_cppcheck_validation

	# Run clang analysis
	run_cppcheck_clang_validation

	# Run infer analysis
	run_infer_analysis

	# Run CPD checks
	run_cpd_check
}

main "$@"
